/**
 ****************************************************************************************
 *
 * @file custom_task.h
 *
 * @brief Header file - CUSTOMTASK.
 *
 * Copyright (C) RivieraWaves 2009-2016
 *
 *
 ****************************************************************************************
 */

#ifndef CUSTOM_TASK_H_
#define CUSTOM_TASK_H_

/**
 ****************************************************************************************
 * @addtogroup CUSTOMTASK Task
 * @ingroup CUSTOM
 * @brief Device Information Service Server Task
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include <stdint.h>
#include "rwip_task.h" // Task definitions
#include "custom.h"
#include "prf_types.h"


/*
 * DEFINES
 ****************************************************************************************
 */



/// Messages for Device Information Service Server
enum
{
    ///Set the value of an attribute - Request
    CUSTOM_SET_VALUE_REQ = TASK_FIRST_MSG(TASK_ID_CUSTOM),
    ///Set the value of an attribute - Response
    CUSTOM_SET_VALUE_RSP,
    
    /// Peer device request to get profile attribute value
    CUSTOM_VALUE_REQ_IND,
    /// Peer device confirm value of requested attribute
    CUSTOM_VALUE_CFM,
};

/*
 * API MESSAGES STRUCTURES
 ****************************************************************************************
 */

/// Parameters for the database creation
struct custom_db_cfg
{
    /// Database configuration @see enum custom_features
    uint16_t features;
};


///Set the value of an attribute - Request
struct custom_set_value_req
{
    /// Value to Set
    uint8_t value;
    /// Value length
    uint8_t length;
    /// Value data
    uint8_t data[__ARRAY_EMPTY];
};

///Set the value of an attribute - Response
struct custom_set_value_rsp
{
    /// Value Set
    uint8_t value;
    /// status of the request
    uint8_t status;
};

/// Peer device request to get profile attribute value
struct custom_value_req_ind
{
    /// Requested value
    uint8_t value;
};

/// Peer device  value of requested attribute
struct custom_value_cfm
{
    /// Requested value
    uint8_t value;
    /// Value length
    uint8_t length;
    /// Value data
    uint8_t data[__ARRAY_EMPTY];
};

/// @} CUSTOMTASK
#endif // CUSTOM_TASK_H_
