#include "at_cmd.h"
#include "stdio.h"
#include "string.h"

extern flag_status recv_cmp_flag;
extern char recv_data[];

cmd_rsp_status cmd_result = RSP_WAIT;
cmd_flash_read_status fr_flag = FLASH_IDLE;
uint8_t bt_flash_data = 0x00;

/**
  * @brief  send string to bk3432, it will be request or response.
  * @param  string: contents that will be sent.
  * @retval none
  */
void at_cmd_send(char* string)
{
  cmd_result = RSP_WAIT;
  while(*string)
  {        
    usart_data_transmit(USART3, (uint16_t) *string++);         
    while (usart_flag_get(USART3, USART_TDC_FLAG) == RESET);
  }   
}

/**
  * @brief  get response status.
  * @param  none
  * @retval response status.
  *         it will be RSP_ERROR or RSP_OK.
  */
cmd_rsp_status cmd_rsp_get(char* string)
{
  while(recv_cmp_flag == RESET);
  if(memcmp(recv_data, string, strlen(string)) == 0)
  {
    cmd_result = RSP_OK;
    
    if(fr_flag == FLASH_READING)
    {
      bt_flash_data = flash_data_rsp_to_hex(recv_data + 4);
      fr_flag = FLASH_IDLE;
    }
  }
  else
  {
    cmd_result = RSP_ERROR;
  }
  recv_cmp_flag = RESET;
  memset(recv_data, 0, strlen(recv_data));
  return cmd_result;
}

/**
  * @brief  set baud rate for at command of bk3432.
  * @param  mem: location that will be writen.
  *         it will be REGION_FLASH or REGION_SRAM.
  * @param  baud_rate: communication baud rate.
  *         this parameter can be one of the following values:
  *         if mem is REGION_FLASH
  *         @arg AT_CMD_BAUD_9600
  *         @arg AT_CMD_BAUD_19200
  *         @arg AT_CMD_BAUD_38400
  *         @arg AT_CMD_BAUD_57600
  *         @arg AT_CMD_BAUD_115200
  *         if mem is REGION_SRAM
  *         @arg AT_CMD_BAUDS_9600
  *         @arg AT_CMD_BAUDS_19200
  *         @arg AT_CMD_BAUDS_38400
  *         @arg AT_CMD_BAUDS_57600
  *         @arg AT_CMD_BAUDS_115200
  * @retval none
  */
void at_cmd_baud_rate_set(cmd_region mem, cmd_baud_rate baud_rate)
{
  switch(mem)
  {
    case REGION_SRAM:
    {
      switch(baud_rate)
      {
        case BAUD_RATE_9600:
        {
          at_cmd_send(AT_CMD_BAUDS_9600);
          break;
        }
        case BAUD_RATE_19200:
        {
          at_cmd_send(AT_CMD_BAUDS_19200);
          break;
        }
        case BAUD_RATE_38400:
        {
          at_cmd_send(AT_CMD_BAUDS_38400);
          break;
        }
        case BAUD_RATE_57600:
        {
          at_cmd_send(AT_CMD_BAUDS_57600);
          break;
        }
        case BAUD_RATE_115200:
        default:
        {
          at_cmd_send(AT_CMD_BAUDS_115200);
          break;
        }
      }
      break;
    }        
  }
}

/**
  * @brief  transform ASCII characters to real hex value.
  * @param  rsp: response of reading bk3432 flash.
  * @retval real hex value.
  */
uint8_t flash_data_rsp_to_hex(char* rsp)
{
  uint8_t rsp_idx, hex_idx, hex_result = 0x00;
  char hex_string[] = "0123456789abcdef";
  uint8_t hex_value[] = {0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 
                         0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF};
  
  for(rsp_idx = 0; rsp_idx < 2; rsp_idx++)
  {
    for(hex_idx = 0; hex_idx < strlen(hex_string); hex_idx++)
    {
      if(rsp[rsp_idx] == hex_string[hex_idx])
      {
        hex_result = (hex_result << 4) | hex_value[hex_idx];        
      }
    }
  }
  
  return hex_result;
}
